#!/bin/bash
########################################################################
# build_bazelisk.sh
#
# Build bazelisk and package for UPS.
#
# Adapted from build.sh.template provided by ssibuildshims v2_11_00.
########################################################################

########################################################################
# Non-customize-able boilerplate: skip to "Customize-able functions."
########################################################################

####################################
# Useful variables.
prog=${BASH_SOURCE[0]##*/}

##################
# Local (dumb) ssi_die() function for use until we source ssi_functions.
ssi_die() {
  printf "FATAL ERROR: ${*}\n" 1>&2; exit 1
}
##################

##################
# Basic usage function for use by ensure_buildshims until we have enough
# information to provide something more specific. Do NOT customize!
build_script_usage() {
    cat <<EOF
USAGE: prog <ssi-msg-config-options> [--] <product-dir> <qualifier-args> <maketar>
       prog -h|--help [<product-dir>]

  Build $package and package for UPS.

OPTIONS

  -h|--help

    Help. Supply <product-dir> for specific, detailed information on
    invoking $prog.

  <ssi-msg-config-options>

    Supply <product-dir> for more information.

ARGUMENTS

  <product-dir>

    The top-level UPS directory in which the source code distribution
    was unpacked. When specified with -h|--help, this enables us to
    provide more specific help about invoking $prog.

  <qualifier-args>
  <maketar>

    Supply <product-dir> for more information.

EOF
}
##################

##################
# Make sure we have UPS and ssibuildshims set up.
ensure_ssibuildshims() {
  if ! (( $# )) || [ "${@: -1}" = "-h" ] || [ "${@: -1}" = "--help" ]; then
    build_script_usage
    exit 1
  fi
  # Find <product-dir> from our arguments:
  local arg
  for arg in "${@}"; do
    [[ "$arg" ==  -* ]] && continue;
    product_dir="$arg"; break
  done
  if [ -z "$product_dir" ]; then
    # Bad usage.
    ssi_die "Bad <product-dir> $product_dir\n$(build_script_usage)"
    exit 2
  elif ! { [ -d "$product_dir/.upsfiles" ] && [ -w "$product_dir" ]; }; then
    # Something wrong with <product-dir>.
    ssi_die "<product-dir> not a valid, writable unified UPS directory: " \
      "$product_dir\n$(build_script_usage)"
  fi
  # Promising...
  local abs_product_dir="$(cd "$product_dir" 2>/dev/null 2>&1 && pwd -P)"
  (( $? )) && ssi_die "unable to cd to <product-dir>: $product_dir\n" \
    "$(build_script_usage)"
  # Find our own bootstraps and yank 'em.
  local ssibuildshims_topdir="$product_dir/ssibuildshims/$ssibuildshims_version"
  [ -f "$ssibuildshims_topdir/bin/ssi_functions" ] && \
    source "$ssibuildshims_topdir/bin/ssi_functions" >/dev/null 2>&1 && \
    ensure_ups "$product_dir" || ssi_die "unable to setup UPS"
    setup ssibuildshims $ssibuildshims_version -z "$product_dir" || \
    ssi_die "unable to find ssibuildshims $ssibuildshims_version in
$product_dir and set it up with UPS."
  # Absolute is safer.
  absolute_path -- "$product_dir" product_dir
}
##################

########################################################################
# Customize-able functions.
########################################################################

##################
# Ascertain whether we built and installed correctly. Add more /
# different calls to verify_in_filesystem as appropriate. Arguments
# (e.g. -q) are passed through to verify_in_filesystem.
is_built() { # *C*
  $trace verify_in_filesystem "${@}" -x "$pkgdir/bin/$package"
}
##################

##################
# If a distribution archive is wanted and the installation is successful
# according to is_built(), then make it. Any arguments (e.g. -q) are
# passed through both to is_built() and make_basic_distribution().
#
# If make_basic_distribution() finds that the product is declared
# correctly and attempts to make the distribution, then it will cause us
# to exit with the status code of that operation. If not, it returns an
# error code.
#
# See make_basic_distribution -h for useful options to add to
# mbd_opts(). *C*
maybe_make_distribution() {
  if [ "$maketar" = "tar" ]; then
    # Customize mbd_opts. *C*
    local mbd_opts=(-c)
    $trace is_built "$@" && \
      $trace make_basic_distribution "${@}" "${mbd_opts[@]}"
  fi
}
##################

########################################################################
# Package-specific functions.
########################################################################

# ...

########################################################################
# Main.
########################################################################

####################################
# Useful variables. *C*
trace=maybe_xtrace # Could set to xtrace instead for unconditional.
(( ssi_trace = 1 )) # Trace certain commands by default.
thisdir=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd -P)
####################################

####################################
# Package name and version information. *C*
. $thisdir/version_info.sh || \
  ssi_die "unable to load version info from $thisdir/version_info"
####################################

##################
# Other useful variables. *C*
##################

####################################
# Handle options and arguments.

# Set up ssibuildshims and UPS / product_dir as early as possible.
ensure_ssibuildshims "${@}"

##################
# Now parse options and arguments, setting basequal, extraqual, maketar,
# etc.
#
# Use verify_quals() to parse command line options, verify that the user
# has provided qualifiers supported by this build script, and set
# expected variables such as basequal, extraqual, cqual, etc.
#
# Customize the extra usage text as appropriate. Note that DEBUG_PATCHES
# and VERBOSE_PATCHES are honored by prepare_source() and
# patch_source(). DEBUG_(CONFIG|BUILD|TEST|INSTALL) should be accounted
# for (or not) by the relevant command in this script.
#
# See verify_quals -h for options and examples.
verify_quals \
  --parse --usage-file=- -- "$@" <<EOF || exit # *C*
ENVIRONMENT

  DEBUG_(CONFIG|BUILD|TEST|INSTALL)

    Verbose output from the corresponding step.

  DEBUG_PATCHES
  VERBOSE_PATCHES

    Command tracing and increased verbosity for source unpacking and patching
    operations. DEBUG_PATCHES will cause exit after patches have been applied.
EOF
##################

##################
# Define globals: pkgdir, sourcedir, blddir, fullqual, etc. See
# define_basics -h for steering options.
$trace define_basics IN_SOURCE NO_QUAL TRACK_PROGRESS || exit
##################

####################################
# Declare ourselves privately, and then set ourselves up to allow setup
# of dependencies. *C*
#
# Comment or delete if not required:
$trace fake_declare_product "${ssi_args[@]}" -x -- || exit
####################################

####################################
# Build-only dependencies. *C*

go_version_required=1.10

if ! $trace setup_prod golang --latest={--ge,$go_version_required} --optional; then
  ssi_warn "could not find golang.org Go compiler of sufficiently recent vintage (>= $go_version_required)\nChecking for go executable in PATH"
  go_version="$(go version 2>/dev/null | sed -Ene 's&^.*go([0-9\.]+).*$&\1&p')"
  [ -n "$go_version" ] || ssi_die "Go compiler not found"
  { ! ver_greater $go_version_required $go_version; } || \
    ssi_die "unable to find Go compiler of sufficiently recent vintage ($go_version < $go_version_required)"
  go=go
else
  go=$GOROOT/bin/go
fi

####################################

# Print active products.
echo
ups active
echo

# Possible short-circuit if we're already built and declared.
$trace maybe_make_distribution --silent

ssi_info "building $package for $OS-$plat-$qualdir (flavor $pkgflvr)"

# Ensure directories: pkgdir, sourcedir, blddir.
$trace init_pkg_dirs || exit

####################################
# Prepare source for building. *C*

$trace cd "$sourcedir" \
  || ssi_die "unable to cd to source directory $sourcedir"

# Source tarballs and patches. See prepare_source -h / patch_source -h
# for options.
$trace prepare_source "${ssi_args[@]}" -P "$patchdir" \
  "$tardir/$pkgtarfile" ${patches[*]:+"${patches[@]}"} \
  || exit

# Bail early if DEBUG_PATCHES is set.
[ -n "$DEBUG_PATCHES" ] && exit 1
####################################

####################################
# Configure, build, and install.
#
# Use $blddir/$pkgtarname for in-source builds, $blddir otherwise.

$trace cd "$blddir"

# Make sure things can be deleted after we exit.
trap "[ -d \"$blddir\" ] && chmod -R u+w \"$blddir\"" EXIT

# Configure go machinery.
export GOPATH=`pwd`

# Make our own cache directory.
export XDG_CACHE_HOME=$blddir/bazelisk-build-cache
mkdir -p "$XDG_CACHE_HOME" || \
  ssi_die "unable to make cache directory $XDG_CACHE_HOME"

module=$(sed -Ene 's&^module[[:space:]]+(.*)$&\1&p' "$pkgtarname/go.mod") || \
  ssi_die "unable to ascertain module from $blddir/$pkgtarname/go.mod"

$trace mkdir -p "src/${module%/*}" && \
  $trace mv "$pkgtarname" "src/$module" || \
  ssi_die "unable to relocate $pkgtarname to go-required build area src/$module"

$trace cd "src/$module" || \
  ssi_die "unable to cd to build directory $blddir/src/$module"

machtype_regex='^([^-]+)-.*(linux|darwin)'
if [[ "$MACHTYPE" =~ $machtype_regex ]]; then
  hosttype=${BASH_REMATCH[1]}
  os=${BASH_REMATCH[2]}
else
  hosttype=${HOSTTYPE:-$(uname -m)}
  os=${OSTYPE:-${OS1:-$(uname -s)}}
fi
case $hosttype in
  x86_64)
    hosttype=amd64
    ;;
  arm64|arm_64)
    hosttype=arm64
    ;;
  *)
    ssi_die "unable to classify host type \"$hosttype\" as amd64 or arm64"
esac
case $os in
  *inux*)
    os=linux
    ;;
  *arwin*)
    os=darwin
    ;;
  *)
    ssi_die "unable to classify operating system \"$os\" as linux or darwin"
esac

##################
# Build.
$trace $go build "$module" || ssi_die "unable to bootstrap bazelisk using $go"
$trace ./bazelisk --output_user_root=$XDG_CACHE_HOME/bazel \
       build --config=release //:bazelisk-$os\_$hosttype \
       --workspace_status_command="$patchdir/stamp.sh" || \
  ssi_die "unable to build final static bazelisk executable via bootstrap"

##################
# Install.
$trace mkdir $pkgdir/bin || \
  ssi_die "unable to make directory $pkgdir/bin"
$trace install bazel-out/*-opt-*/bin/bazelisk-$os\_$hosttype $pkgdir/bin/bazelisk || \
  ssi_die "cannot install "bazel-out/*-opt-*/bin/bazelisk-$os\_$hosttype" as $pkgdir/bin/bazelisk"
$trace ln $pkgdir/bin/bazel{isk,} || \
  ssi_die "cannot create link bazel -> bazelisk in $pkgdir/bin"

##################
# Check.
bazelisk_version="$($pkgdir/bin/bazel version --gnu_format 2>/dev/null | head -1)"
ssi_vinfo "bazelisk executable reports $bazelisk_version"
[ "$bazelisk_version" = "Bazelisk v$origpkgdotver" ] || \
  ssi_die "version not properly encoded into bazelisk executable:\n" \
          "expected \"Bazelisk v$origpkgdotver\", got \"$bazelisk_version\""

##################
# Done.
ssi_info "finished building $package $pkgver"

##################
# Declare and set up the installed package. *C*
#
# Add -c for current declaration if required.
$trace declare_product -c -x -- || exit
##################

#################
# Verify installation and make distribution archive. *C*
#
# To customize verification, see is_built(), since it is also used by
# maybe_make_distribution(). See the documentation for those functions
# (above) for customization opportunities.

# Add a message to ssi_build_failed if appropriate.
$trace is_built && $trace ssi_build_completed && \
  ssi_info "$package is installed at $pkgdir" && \
  $trace maybe_make_distribution || \
  ssi_build_failed # *C*
########################################################################
# Nothing beyond this point.
########################################################################
